import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, NgModule, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux';
var METADATA_KEY$2 = Symbol('@ReduxActionController');
var METADATA_DEFAULT$2 = {
    actionPrefix: '',
};
/**
 * @param {?} actionPrefix
 * @return {?}
 */
function ReduxActionController(actionPrefix) {
    return function (constructor) {
        var /** @type {?} */ metadata = { actionPrefix: actionPrefix };
        Reflect['defineMetadata'](METADATA_KEY$2, metadata, constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionControllerMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$2, Reflect['getMetadata'](METADATA_KEY$2, target));
}
/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(function (store) { return store.dispatch(action); });
}
var METADATA_KEY$1 = Symbol('@ReduxActionController');
var METADATA_DEFAULT$1 = {
    methodType: '',
    targetConstructor: null,
};
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return function (target, propertyKey, descriptor) {
        var /** @type {?} */ methodType = customType || propertyKey;
        var /** @type {?} */ method = descriptor.value;
        var /** @type {?} */ proxyFunction = function () {
            var /** @type {?} */ metadata = getReduxActionControllerMetadata(target.constructor);
            var /** @type {?} */ type = getActionType(metadata.actionPrefix, methodType);
            var /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then(function (p) { return dispatch({ payload: p, type: type }); });
            }
            else {
                dispatch({ payload: payload, type: type });
            }
            return payload;
        };
        Reflect['defineMetadata'](METADATA_KEY$1, {
            methodType: methodType,
            targetConstructor: target.constructor,
        }, proxyFunction);
        descriptor.value = proxyFunction;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$1, Reflect['getMetadata'](METADATA_KEY$1, target));
}
/**
 * @param {?} type
 * @return {?}
 */
function getActionTypeByFunction(type) {
    var _a = getReduxActionMetadata(type), methodType = _a.methodType, targetConstructor = _a.targetConstructor;
    var actionPrefix = getReduxActionControllerMetadata(targetConstructor).actionPrefix;
    return getActionType(actionPrefix, methodType);
}
/**
 * @param {...?} rest
 * @return {?}
 */
function getActionType() {
    var rest = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        rest[_i] = arguments[_i];
    }
    return rest.filter(function (i) { return i.length > 0; }).join('/');
}
var METADATA_KEY$3 = Symbol('@ReduxActionController');
var METADATA_DEFAULT$3 = {
    reducers: [],
};
/**
 * @template S, P
 * @param {?} types
 * @return {?}
 */
function ReduxReducer(types) {
    return function (target, propertyKey) {
        var /** @type {?} */ metadata = getReduxReducerClassMetadata(target.constructor);
        metadata.reducers.push({ types: types, reducer: target[propertyKey].bind(target) });
        Reflect['defineMetadata'](METADATA_KEY$3, metadata, target.constructor);
        // if (typeof target === 'object') {
        //   target[ propertyKey ].bind(target);
        // }
        return target[propertyKey];
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxReducerClassMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$3, Reflect['getMetadata'](METADATA_KEY$3, target));
}
var ReduxRegistryReducerItem = /** @class */ (function () {
    function ReduxRegistryReducerItem() {
    }
    return ReduxRegistryReducerItem;
}());
var ReduxRegistry = /** @class */ (function () {
    function ReduxRegistry() {
    }
    /**
     * @param {?} store
     * @return {?}
     */
    ReduxRegistry.registerStore = function (store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    };
    /**
     * @param {?} stateName
     * @param {?} types
     * @param {?} reducer
     * @return {?}
     */
    ReduxRegistry.registerReducer = function (stateName, types, reducer) {
        if (Array.isArray(types)) {
            types.forEach(function (type) {
                ReduxRegistry._reducers.push({
                    reducer: reducer,
                    stateName: stateName,
                    type: type,
                });
            });
        }
        else {
            ReduxRegistry._reducers.push({
                reducer: reducer,
                stateName: stateName,
                type: /** @type {?} */ (types),
            });
        }
    };
    /**
     * @param {?} stateType
     * @return {?}
     */
    ReduxRegistry.registerState = function (stateType) {
        ReduxRegistry.getStore().then(function (store) {
            var /** @type {?} */ stateConfig = getReduxStateMetadata(stateType);
            var /** @type {?} */ state = new stateType();
            Promise.resolve(state.initialize()).then(function (initialState) {
                stateConfig.reducers
                    .map(function (reducer) { return getReduxReducerClassMetadata(reducer.constructor); })
                    .forEach(function (metadata) {
                    metadata.reducers.forEach(function (reducer) {
                        ReduxRegistry.registerReducer(stateConfig.name, reducer.types, reducer.reducer);
                    });
                });
                store.dispatch({
                    payload: {
                        initialState: initialState,
                        stateName: stateConfig.name,
                    },
                    type: ReduxRegistry.ACTION_REGISTER_MODULE,
                });
            });
        });
    };
    /**
     * @param {?} type
     * @return {?}
     */
    ReduxRegistry.getReducerItemsByType = function (type) {
        return ReduxRegistry._reducers
            .filter(function (reducerItem) { return getActionTypeByFunction(reducerItem.type) === type; });
    };
    /**
     * @return {?}
     */
    ReduxRegistry.getStore = function () {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    };
    return ReduxRegistry;
}());
ReduxRegistry.ACTION_REGISTER_MODULE = ReduxRegistry.name + "://registerModule";
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = [];
var METADATA_KEY = Symbol('@ReduxState');
var METADATA_DEFAULT = {
    name: '',
    reducers: [],
};
/**
 * @param {?} config
 * @return {?}
 */
function ReduxState(config) {
    return function (constructor) {
        Reflect['defineMetadata'](METADATA_KEY, config, constructor);
        ReduxRegistry.registerState(constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxStateMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT, Reflect['getMetadata'](METADATA_KEY, target));
}
/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function selectByState(state, selector, context) {
    if (!selector.startsWith('/')) {
        if (!context) {
            throw new Error('Relative selectors need to have a state context!');
        }
        selector = "/" + getReduxStateMetadata(context).name + "/" + selector;
    }
    var /** @type {?} */ selectorArray = selector.split('/');
    return selectorArray
        .filter(function (propertyKey) { return propertyKey !== ''; })
        .reduce(function (previousValue, propertyKey) {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function select(selector, context) {
    var /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(function (store) {
        subject.next(selectByState(store.getState(), selector, context));
        store.subscribe(function () { return subject.next(selectByState(store.getState(), selector, context)); });
    });
    return subject.asObservable();
}
var REDUX_ROOT_MODULE_CONFIG = new InjectionToken('REDUX_ROOT_MODULE_CONFIG');
var REDUX_CHILD_MODULE_CONFIG = new InjectionToken('REDUX_CHILD_MODULE_CONFIG');
var ReduxSelectPipe = /** @class */ (function () {
    /**
     * @param {?} moduleConfig
     */
    function ReduxSelectPipe(moduleConfig) {
        this.moduleConfig = moduleConfig;
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    ReduxSelectPipe.prototype.transform = function (selector) {
        return select(selector, this.moduleConfig.state);
    };
    return ReduxSelectPipe;
}());
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_CHILD_MODULE_CONFIG,] },] },
]; };
/**
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function ReduxSelect(selector, context) {
    return function (target, propertyKey) {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector, context),
        });
    };
}
var rootReducer = function (state, action) {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        var _a = action.payload, stateName = _a.stateName, initialState = _a.initialState;
        return Object.assign({}, state, (_b = {},
            _b[stateName] = initialState,
            _b));
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce(function (stateToReduce, reducerItem) {
        var stateName = reducerItem.stateName, reducer = reducerItem.reducer;
        return Object.assign({}, stateToReduce, (_a = {},
            _a[stateName] = reducer(stateToReduce[stateName], action),
            _a));
        var _a;
    }, state);
    var _b;
};
var ReduxRootModule = /** @class */ (function () {
    /**
     * @param {?=} config
     */
    function ReduxRootModule(config) {
        if (config === void 0) { config = null; }
        config = Object.assign({
            store: null,
        }, config || {});
        ReduxRegistry.registerStore(config.store || createStore(rootReducer, {}));
    }
    return ReduxRootModule;
}());
ReduxRootModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                ],
            },] },
];
/**
 * @nocollapse
 */
ReduxRootModule.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_ROOT_MODULE_CONFIG,] },] },
]; };
var ReduxChildModule = /** @class */ (function () {
    function ReduxChildModule() {
    }
    return ReduxChildModule;
}());
ReduxChildModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                exports: [],
                imports: [
                    CommonModule,
                ],
            },] },
];
/**
 * @nocollapse
 */
ReduxChildModule.ctorParameters = function () { return []; };
var ReduxModule = /** @class */ (function () {
    function ReduxModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ReduxModule.forChild = function (config) {
        return {
            ngModule: ReduxChildModule,
            providers: [
                { provide: REDUX_CHILD_MODULE_CONFIG, useValue: config },
            ],
        };
    };
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxModule.forRoot = function (config) {
        return {
            ngModule: ReduxRootModule,
            providers: [
                { provide: REDUX_ROOT_MODULE_CONFIG, useValue: config },
            ],
        };
    };
    return ReduxModule;
}());
/**
 * Generated bundle index. Do not edit.
 */
export { selectByState, select, ReduxSelectPipe, ReduxAction, getReduxActionMetadata, getActionTypeByFunction, getActionType, ReduxActionController, getReduxActionControllerMetadata, ReduxReducer, getReduxReducerClassMetadata, ReduxSelect, ReduxState, getReduxStateMetadata, REDUX_ROOT_MODULE_CONFIG, REDUX_CHILD_MODULE_CONFIG, ReduxRootModule, ReduxChildModule, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.es5.js.map
